@@ -1,6 +1,6 @@
 from . import bruker
 from . import convert
-# from . import glue
+from . import nmrml
 from . import pipe
 from . import rnmrtk
 from . import simpson
