@@ -51,7 +51,7 @@ def create_data(data):
 
 # universal dictionary functions
 
-def guess_udic(dic, data):
+def guess_udic(dic, data, strip_fake=False):
     """
     Guess parameters of universal dictionary from dic, data pair.
 
@@ -61,6 +61,12 @@ def guess_udic(dic, data):
         Dictionary of Bruker parameters.
     data : ndarray
         Array of NMR data.
+    strip_fake: bool
+        If data is proceed (i.e. read using `bruker.read_pdata`) and the Bruker
+        processing parameters STSI and/or STSR are set, the returned sweep
+        width and carrier frequencies is changed to values that are incorrect
+        but instead can are intended to trick the normal unit_conversion object
+        into producing the correct result.
 
     Returns
     -------
@@ -68,7 +74,7 @@ def guess_udic(dic, data):
         Universal dictionary of spectral parameters.
 
     """
-    # XXX if pprog, acqus are in dic use them
+    # TODO if pprog, acqus, procs are in dic use them better.
 
     # create an empty universal dictionary
     udic = fileiobase.create_blank_udic(data.ndim)
@@ -79,13 +85,13 @@ def guess_udic(dic, data):
 
         # try to add additional parameter from acqus dictionary keys
         try:
-            add_axis_to_udic(udic, dic, b_dim)
+            add_axis_to_udic(udic, dic, b_dim, strip_fake)
         except:
             warn("Failed to determine udic parameters for dim: %i" % (b_dim))
     return udic
 
 
-def add_axis_to_udic(udic, dic, udim):
+def add_axis_to_udic(udic, dic, udim, strip_fake):
     """
     Add axis parameters to a udic.
 
@@ -97,26 +103,52 @@ def add_axis_to_udic(udic, dic, udim):
         Bruker dictionary used to determine axes parameters.
     dim : int
         Universal dictionary dimension to update.
+    strip_fake: bool
+        See `bruker.guess_udic`
 
     """
     # This could still use some work
     b_dim = udic['ndim'] - udim - 1  # last dim
     acq_file = "acqu" + str(b_dim + 1) + "s"
+    pro_file = "proc" + str(b_dim + 1) + "s"
 
+    # Because they're inconsistent,..
     if acq_file == "acqu1s":
-        acq_file = "acqus"   # Because they're inconsistent,...
+        acq_file = "acqus"
 
-    udic[udim]["sw"] = dic[acq_file]["SW_h"]
+    if pro_file == "proc1s":
+        pro_file = "procs"
+
+    sw = dic[acq_file]["SW_h"]
     udic[udim]["label"] = dic[acq_file]["NUC1"]
-    udic[udim]["car"] = dic[acq_file]["O1"]
 
-    if "acqus" in dic and dic["acqus"]["NUC2"] == "15N":
-        # Gyromagnetic ratio of 15N is negative
-        udic[udim]["obs"] = (dic[acq_file]["BF1"] -
-                             dic[acq_file]["O1"] / 1.e6)
-    else:
-        udic[udim]["obs"] = (dic[acq_file]["BF1"] +
-                             dic[acq_file]["O1"] / 1.e6)
+    try:
+        obs = dic[pro_file]["SF"]
+        car = (dic[acq_file]["SFO1"]-obs) * 1e6
+
+    except KeyError:
+        warn('The chemical shift referencing was not corrected for "sr".')
+        obs = dic[acq_file]["SFO1"]
+        car = dic[acq_file]["O1"]
+
+    if strip_fake:
+        try:
+
+            # Temporary parameters
+            w = sw/float(dic[pro_file]["FTSIZE"])
+            d = (w * dic[pro_file]["STSR"]) + (w * dic[pro_file]["STSI"]/2.0)
+
+            # Fake car frequency
+            car -= (d-(sw/2.0))
+
+            # Fake sw frequency
+            sw = w * dic[pro_file]["STSI"]
+        except KeyError:
+            pass
+
+    udic[udim]["sw"] = sw
+    udic[udim]["car"] = car
+    udic[udim]["obs"] = obs
 
     if acq_file == "acqus":
         if dic['acqus']['AQ_mod'] == 0:     # qf
@@ -219,7 +251,7 @@ def create_acqus_dic(adic, direct=False):
 
 def read(dir=".", bin_file=None, acqus_files=None, pprog_file=None,
          shape=None, cplex=None, big=None, read_pulseprogram=True,
-         read_acqus=True):
+         read_acqus=True, procs_files=None, read_procs=True):
     """
     Read Bruker files from a directory.
 
@@ -247,6 +279,11 @@ def read(dir=".", bin_file=None, acqus_files=None, pprog_file=None,
         True to read pulse program, False prevents reading.
     read_acqus : bool, optional
         True to read acqus files(s), False prevents reading.
+    procs_files : list, optional
+        List of filename(s) of procs parameter files in directory. None uses
+        standard files.
+    read_procs : bool, optional
+        True to read procs files(s), False prevents reading.
 
     Returns
     -------
@@ -265,32 +302,45 @@ def read(dir=".", bin_file=None, acqus_files=None, pprog_file=None,
     if os.path.isdir(dir) is not True:
         raise IOError("directory %s does not exist" % (dir))
 
+    # Take a shot at reading the procs file
+    if read_procs:
+        dic = read_procs_file(dir, procs_files)
+    else:
+        # create an empty dictionary
+        dic = dict()
+
     # determind parameter automatically
     if bin_file is None:
         if os.path.isfile(os.path.join(dir, "fid")):
             bin_file = "fid"
         elif os.path.isfile(os.path.join(dir, "ser")):
             bin_file = "ser"
-        else:
-            raise IOError("No Bruker binary file could be found in %s" % (dir))
 
-    if acqus_files is None:
-        acqus_files = []
-        for f in ["acqus", "acqu2s", "acqu3s", "acqu4s"]:
-            if os.path.isfile(os.path.join(dir, f)):
-                acqus_files.append(f)
+        # Look two directory levels lower.
+        elif os.path.isdir(os.path.dirname(os.path.dirname(dir))):
+
+            # ! change the dir
+            dir = os.path.dirname(os.path.dirname(dir))
+
+            if os.path.isfile(os.path.join(dir, "fid")):
+                bin_file = "fid"
+            elif os.path.isfile(os.path.join(dir, "ser")):
+                bin_file = "ser"
+            else:
+                mesg = "No Bruker binary file could be found in %s"
+                raise IOError(mesg % (dir))
+        else:
+            mesg = "No Bruker binary file could be found in %s"
+            raise IOError(mesg % (dir))
+
+    if read_acqus:
+        # read the acqus_files and add to the dictionary
+        acqus_dic = read_acqus_file(dir, acqus_files)
+        dic = _merge_dict(dic, acqus_dic)
 
     if pprog_file is None:
         pprog_file = "pulseprogram"
 
-    # create an empty dictionary
-    dic = dict()
-
-    # read the acqus_files and add to the dictionary
-    if read_acqus:
-        for f in acqus_files:
-            dic[f] = read_jcamp(os.path.join(dir, f))
-
     # read the pulse program and add to the dictionary
     if read_pulseprogram:
         try:
@@ -306,7 +356,7 @@ def read(dir=".", bin_file=None, acqus_files=None, pprog_file=None,
         gshape, gcplex = guess_shape(dic)
         if gcplex is True:    # divide last dim by 2 if complex
             t = list(gshape)
-            t[-1] = t[-1] / 2
+            t[-1] = t[-1] // 2
             gshape = tuple(t)
     if shape is None:
         shape = gshape
@@ -330,7 +380,7 @@ def read(dir=".", bin_file=None, acqus_files=None, pprog_file=None,
 
 def read_lowmem(dir=".", bin_file=None, acqus_files=None, pprog_file=None,
                 shape=None, cplex=None, big=None, read_pulseprogram=True,
-                read_acqus=True):
+                read_acqus=True, procs_files=None, read_procs=True):
     """
     Read Bruker files from a directory using minimal amounts of memory.
 
@@ -351,7 +401,14 @@ def read_lowmem(dir=".", bin_file=None, acqus_files=None, pprog_file=None,
     """
 
     if os.path.isdir(dir) is not True:
-        raise IOError("Directory %s does not exist" % (dir))
+        raise IOError("directory %s does not exist" % (dir))
+
+    # Take a shot at reading the procs file
+    if read_procs:
+        dic = read_procs_file(dir, procs_files)
+    else:
+        # create an empty dictionary
+        dic = dict()
 
     # determind parameter automatically
     if bin_file is None:
@@ -359,26 +416,32 @@ def read_lowmem(dir=".", bin_file=None, acqus_files=None, pprog_file=None,
             bin_file = "fid"
         elif os.path.isfile(os.path.join(dir, "ser")):
             bin_file = "ser"
-        else:
-            raise IOError("no Bruker binary file could be found in %s" % (dir))
 
-    if acqus_files is None:
-        acqus_files = []
-        for f in ["acqus", "acqu2s", "acqu3s", "acqu4s"]:
-            if os.path.isfile(os.path.join(dir, f)):
-                acqus_files.append(f)
+        # Look two directory levels lower.
+        elif os.path.isdir(os.path.dirname(os.path.dirname(dir))):
+
+            # ! change the dir
+            dir = os.path.dirname(os.path.dirname(dir))
+
+            if os.path.isfile(os.path.join(dir, "fid")):
+                bin_file = "fid"
+            elif os.path.isfile(os.path.join(dir, "ser")):
+                bin_file = "ser"
+            else:
+                mesg = "No Bruker binary file could be found in %s"
+                raise IOError(mesg % (dir))
+        else:
+            mesg = "No Bruker binary file could be found in %s"
+            raise IOError(mesg % (dir))
+
+    if read_acqus:
+        # read the acqus_files and add to the dictionary
+        acqus_dic = read_acqus_file(dir, acqus_files)
+        dic = _merge_dict(dic, acqus_dic)
 
     if pprog_file is None:
         pprog_file = "pulseprogram"
 
-    # create an empty dictionary
-    dic = dict()
-
-    # read the acqus_files and add to the dictionary
-    if read_acqus:
-        for f in acqus_files:
-            dic[f] = read_jcamp(os.path.join(dir, f))
-
     # read the pulse program and add to the dictionary
     if read_pulseprogram:
         dic["pprog"] = read_pprog(os.path.join(dir, pprog_file))
@@ -413,6 +476,70 @@ def read_lowmem(dir=".", bin_file=None, acqus_files=None, pprog_file=None,
     return dic, data
 
 
+def read_acqus_file(dir='.', acqus_files=None):
+    """
+    Read Bruker acquisition files from a directory.
+
+    Parameters
+    ----------
+    dir : str
+        Directory to read from.
+    acqus_files : list, optional
+        List of filename(s) of acqus parameter files in directory. None uses
+        standard files.
+
+    Returns
+    -------
+    dic : dict
+        Dictionary of Bruker parameters.
+    """
+    if acqus_files is None:
+        acqus_files = []
+        for f in ["acqus", "acqu2s", "acqu3s", "acqu4s"]:
+            if os.path.isfile(os.path.join(dir, f)):
+                acqus_files.append(f)
+
+    # create an empty dictionary
+    dic = dict()
+
+    # read the acqus_files and add to the dictionary
+    for f in acqus_files:
+        dic[f] = read_jcamp(os.path.join(dir, f))
+    return dic
+
+
+def read_procs_file(dir='.', procs_files=None):
+    """
+    Read Bruker processing files from a directory.
+
+    Parameters
+    ----------
+    dir : str
+        Directory to read from.
+    procs_files : list, optional
+        List of filename(s) of procs parameter files in directory. None uses
+        standard files.
+
+    Returns
+    -------
+    dic : dict
+        Dictionary of Bruker parameters.
+    """
+    if procs_files is None:
+        procs_files = []
+        for f in ["procs", "proc2s", "proc3s", "proc4s"]:
+            if os.path.isfile(os.path.join(dir, f)):
+                procs_files.append(f)
+
+    # create an empty dictionary
+    dic = dict()
+
+    # read the acqus_files and add to the dictionary
+    for f in procs_files:
+        dic[f] = read_jcamp(os.path.join(dir, f))
+    return dic
+
+
 def write(dir, dic, data, bin_file=None, acqus_files=None, pprog_file=None,
           overwrite=False, big=None, write_prog=True, write_acqus=True):
     """
@@ -696,11 +823,15 @@ def guess_shape(dic):
 # Bruker processed binary (1r, 1i, 2rr, 2ri, etc) reading
 
 def read_pdata(dir=".", bin_files=None, procs_files=None, read_procs=True,
-               shape=None, submatrix_shape=None, all_components=False,
-               big=None):
+               acqus_files=None, read_acqus=True, scale_data=True, shape=None,
+               submatrix_shape=None, all_components=False, big=None):
     """
     Read processed Bruker files from a directory.
 
+    In Topspin and other programs this data is typically scaled by dividing by
+    2 ** -NC_proc where NC_proc is defined in the procs file.  This scaling
+    can be accomplished by setting the scale_data parameter to True.
+
     Parameters
     ----------
     dir : str
@@ -711,8 +842,17 @@ def read_pdata(dir=".", bin_files=None, procs_files=None, read_procs=True,
     procs_files : list, optional
         List of filename(s) of procs parameter files in directory. None uses
         standard files.
-    read_procss : bool, optional
+    read_procs : bool, optional
         True to read procs files(s), False prevents reading.
+    acqus_files : list, optional
+        List of filename(s) of acqus parameter files in directory. None uses
+        standard files.
+    read_acqus : bool, optional
+        True to read acqus files(s), False prevents reading.
+    scale_data : bool, optional
+        True, the default, to apply scaling defined in the procs file.  The
+        data should almost always be scaled. False, returns the
+        data as it appears in the file.
     shape : tuple, optional
         Shape of resulting data.  None will guess the shape from the
         parameters in the procs file(s).
@@ -720,7 +860,7 @@ def read_pdata(dir=".", bin_files=None, procs_files=None, read_procs=True,
         Shape of submatrix for 2D+ data.  None will guess the shape from
         the metadata in the procs file(s).
     all_components : bool
-        True to return all a list of all components, False returns just the
+        True to return a list of all components, False returns just the
         all real component (1r, 2rr, 3rrr, etc).
     big : bool or None, optional
         Endiness of binary file. True of big-endian, False for little-endian,
@@ -766,19 +906,25 @@ def read_pdata(dir=".", bin_files=None, procs_files=None, read_procs=True,
         else:
             raise IOError("No Bruker binary file could be found in %s" % (dir))
 
-    if procs_files is None:
-        procs_files = []
-        for f in ["procs", "proc2s", "proc3s", "proc4s"]:
-            if os.path.isfile(os.path.join(dir, f)):
-                procs_files.append(f)
-
-    # create an empty dictionary
-    dic = dict()
-
-    # read the acqus_files and add to the dictionary
     if read_procs:
-        for f in procs_files:
-            dic[f] = read_jcamp(os.path.join(dir, f))
+        # read the acqus_files and add to the dictionary
+        dic = read_procs_file(dir, procs_files)
+    else:
+        # create an empty dictionary
+        dic = dict()
+
+    if read_acqus:
+        # If acqus files were not listed check in the usual place.
+        acqus_dir = os.path.dirname(os.path.dirname(dir))
+        if acqus_files is not None:
+            acqus_dic = read_acqus_file(dir, acqus_files)
+            # Merge the two dicts.
+            dic = _merge_dict(dic, acqus_dic)
+
+        elif os.path.isdir(acqus_dir):
+            acqus_dic = read_acqus_file(acqus_dir)
+            # Merge the two dicts.
+            dic = _merge_dict(dic, acqus_dic)
 
     # determind shape and complexity for direct dim if needed
     if submatrix_shape is None or shape is None:
@@ -808,12 +954,40 @@ def read_pdata(dir=".", bin_files=None, procs_files=None, read_procs=True,
                               shape, submatrix_shape, big)[1]
             for f in bin_files]
 
+    # scale data if requested
+    if scale_data:
+        data = [scale_pdata(dic, d) for d in data]
+
     if len(data) == 1:
         return dic, data[0]
     else:
         return dic, data
 
 
+def scale_pdata(dic, data):
+    """
+    Scale Bruker processed data using parameters from the procs file.
+
+    Parameters
+    ----------
+    dic : dict
+        Dictionary of Bruker parameters.
+    data : ndarray
+        Array of NMR data.
+
+    Returns
+    -------
+    sdata : array
+        Scaled data.
+    """
+    try:
+        scale = np.power(2., -float(dic['procs']['NC_proc']))
+    except KeyError:
+        warn('Unable to scale data, returning unscaled data')
+        scale = 1
+    return data / scale
+
+
 def guess_shape_and_submatrix_shape(dic):
     """
     Guess the data shape and the shape of the processed data submatrix.
@@ -903,7 +1077,7 @@ def read_pdata_binary(filename, shape=None, submatrix_shape=None, big=True):
     else:
         try:
             data = reorder_submatrix(data, shape, submatrix_shape)
-            return dic, reorder_submatrix(data, shape, submatrix_shape)
+            return dic, data
         except:
             warn('unable to reorder data')
             return dic, data
@@ -951,7 +1125,7 @@ def reorder_submatrix(data, shape, submatrix_shape):
 
 def read_binary(filename, shape=(1), cplex=True, big=True):
     """
-    Read Bruker binary data from file and return dic,data pair
+    Read Bruker binary data from file and return dic,data pair.
 
     If data cannot be reshaped as described a 1D representation of the data
     will be returned after printing a warning message.
@@ -1153,14 +1327,14 @@ class bruker_nd(fileiobase.data_nd):
 
     def __fcopy__(self, order):
         """
-        Create a copy
+        Create a copy.
         """
         n = bruker_nd(self.filename, self.fshape, self.cplex, self.big, order)
         return n
 
     def __fgetitem__(self, slices):
         """
-        return ndarray of selected values
+        Return ndarray of selected values.
 
         slices is a well formatted tuple of slices
         """
@@ -1208,7 +1382,7 @@ class bruker_nd(fileiobase.data_nd):
 
 def get_data(f, big):
     """
-    Get binary data from file object with given endiness
+    Get binary data from file object with given endiness.
     """
     if big:
         return np.frombuffer(f.read(), dtype='>i4')
@@ -1218,7 +1392,7 @@ def get_data(f, big):
 
 def put_data(f, data, big=True):
     """
-    Put data to file object with given endiness
+    Put data to file object with given endiness.
     """
     if big:
         f.write(data.astype('>i4').tostring())
@@ -1229,7 +1403,7 @@ def put_data(f, data, big=True):
 
 def get_trace(f, num_points, big):
     """
-    Get trace of num_points from file with given endiness
+    Get trace of num_points from file with given endiness.
     """
     if big:
         bsize = num_points * np.dtype('>i4').itemsize
@@ -1251,7 +1425,7 @@ def complexify_data(data):
 
 def uncomplexify_data(data_in):
     """
-    Uncomplexify data (pack real,imag) into a int32 array
+    Uncomplexify data (pack real,imag) into a int32 array.
     """
     size = list(data_in.shape)
     size[-1] = size[-1] * 2
@@ -1365,7 +1539,7 @@ bruker_dsp_table = {
 }
 
 
-def remove_digital_filter(dic, data):
+def remove_digital_filter(dic, data, truncate=True):
     """
     Remove the digital filter from Bruker data.
 
@@ -1375,6 +1549,11 @@ def remove_digital_filter(dic, data):
         Dictionary of Bruker parameters.
     data : ndarray
         Array of NMR data to remove digital filter from.
+    truncate : bool, optional
+        True to truncate the phase shift prior to removing the digital filter.
+        This typically produces a better looking spectrum but may remove
+        useful data.  False uses a non-truncated phase.
+
 
     Returns
     -------
@@ -1402,10 +1581,10 @@ def remove_digital_filter(dic, data):
     else:
         grpdly = dic['acqus']['GRPDLY']
 
-    return rm_dig_filter(data, decim, dspfvs, grpdly)
+    return rm_dig_filter(data, decim, dspfvs, grpdly, truncate)
 
 
-def rm_dig_filter(data, decim, dspfvs, grpdly=0):
+def rm_dig_filter(data, decim, dspfvs, grpdly=0, truncate_grpdly=True):
     """
     Remove the digital filter from Bruker data.
 
@@ -1418,8 +1597,13 @@ def rm_dig_filter(data, decim, dspfvs, grpdly=0):
     dspfvs : int
         Firmware version (Bruker DSPFVS parameter).
     grpdly : float, optional
-        Group delay. (Bruker GRPDLY parameter). When non-zero decom and
+        Group delay. (Bruker GRPDLY parameter). When non-zero decim and
         dspfvs are ignored.
+    truncate_grpdly : bool, optional
+        True to truncate the value of grpdly provided or determined from
+        the decim and dspfvs parameters before removing the digital filter.
+        This typically produces a better looking spectrum but may remove useful
+        data.  False uses a non-truncated grpdly value.
 
     Returns
     -------
@@ -1468,6 +1652,9 @@ def rm_dig_filter(data, decim, dspfvs, grpdly=0):
                 raise ValueError("decim not in lookup table")
             phase = bruker_dsp_table[dspfvs][decim]
 
+    if truncate_grpdly:     # truncate the phase
+        phase = np.floor(phase)
+
     # and the number of points to remove (skip) and add to the beginning
     skip = int(np.floor(phase + 2.))    # round up two integers
     add = int(max(skip - 6, 0))           # 6 less, or 0
@@ -1544,7 +1731,7 @@ def read_jcamp(filename):
 
 def parse_jcamp_line(line, f):
     """
-    Parse a single JCAMP-DX line
+    Parse a single JCAMP-DX line.
 
     Extract the Bruker parameter name and value from a line from a JCAMP-DX
     file.  This may entail reading additional lines from the fileobj f if the
@@ -1602,7 +1789,7 @@ def parse_jcamp_value(text):
 
 def write_jcamp(dic, filename, overwrite=False):
     """
-    Write a Bruker JCAMP-DX file from a dictionary
+    Write a Bruker JCAMP-DX file from a dictionary.
 
     Written file will differ slightly from Bruker's JCAMP-DX files in that all
     multi-value parameters will be written on multiple lines. Bruker is
@@ -1882,7 +2069,6 @@ def read_pprog(filename):
     # create the output dictionary
     dic = {"var": var, "incr": incr, "loop": loop, "phase": phase,
            "ph_extra": ph_extra}
-
     return dic
 
 
@@ -1936,5 +2122,10 @@ def write_pprog(filename, dic, overwrite=False):
 
     # close the file
     f.close()
-
     return
+
+
+def _merge_dict(a, b):
+    c = a.copy()
+    c.update(b)
+    return c
