@@ -181,13 +181,12 @@ def make_uc(dic, data, dim=-1):
     if obs == 0.0:
         obs = 1.0
 
-    car = dic[fn + "CAR"] * obs
-    # NMRPipe keeps the carrier constant during extractions storing the
-    # location of this point as CENTER.  This may not be the actual "center" of
-    # the spectrum and may not even be a valid index in that dimension. We need
-    # to re-center the carrier value so that actually represents the
-    # frequency of the central point in the dimension.
-    car = car + sw / size * (dic[fn + "CENTER"] - 1. - size / 2.)
+    # calculate the carrier from the origin, the left most point which has a
+    # frequency of CAR*OBS - SW * (N/2 - 1) / 2,
+    # see Fig 3.1 on p.36 of Hoch and Stern
+    # The carried should have units on MHz so solve the above for CAR*OBS
+    orig = dic[fn + "ORIG"]
+    car = orig + sw / 2. - sw / size
     return fileiobase.unit_conversion(size, cplx, sw, obs, car)
 
 ############################
@@ -236,7 +235,7 @@ def guess_udic(dic, data):
         udic[i]["size"] = data.shape[i]     # size from data shape
 
         # determind NMRPipe axis name
-        fn = ["FDF2", "FDF1", "FDF3", "FDF4"][(data.ndim - 1) - i]
+        fn = "FDF" + str(int(dic["FDDIMORDER"][data.ndim - 1 - i]))
 
         # directly corresponding
         udic[i]["sw"] = dic[fn + "SW"]
@@ -1120,7 +1119,7 @@ def write_slice_3D(filemask, dic, data, shape, slices):
 
         # unpack into rdata,[idata] depending on quadrature
         if data.dtype == 'complex64':
-            h = mdata.shape[-1] / 2.0
+            h = mdata.shape[-1] // 2.0
             rdata = mdata[..., :h]
             idata = mdata[..., h:]
         else:
@@ -1493,7 +1492,7 @@ def unappend_data(data):
     Data should have imaginary data vector appended to real data vector
 
     """
-    h = data.shape[-1] / 2.0
+    h = int(data.shape[-1] / 2)
     return np.array(data[..., :h] + data[..., h:] * 1.j, dtype="complex64")
 
 
@@ -1791,7 +1790,11 @@ class pipe_3d(fileiobase.data_nd):
 
         # find the length of the third dimension
         f3 = "FDF" + str(int(dic["FDDIMORDER3"]))
-        lenZ = int(dic[f3 + "SIZE"])
+        quadrature_factor = [2, 1][int(dic[f3 + 'QUADFLAG'])]
+        if dic[f3 + 'FTFLAG']:
+            lenZ = int(dic[f3 + 'FTSIZE'] * quadrature_factor)
+        else:
+            lenZ = int(dic[f3 + 'TDSIZE'] * quadrature_factor)
         fshape.insert(0, lenZ)   # insert as leading size of fshape
 
         # check that all files exist if fcheck is set
@@ -1802,7 +1805,7 @@ class pipe_3d(fileiobase.data_nd):
 
         # check last axis quadrature
         fn = "FDF" + str(int(dic["FDDIMORDER1"]))
-        if dic[fn + "QUADFLAG"] == 1.0:
+        if dic[fn + "FTFLAG"] == 1.0:
             self.cplex = False
             self.dtype = np.dtype('float32')
         else:
@@ -1991,12 +1994,20 @@ class pipe_4d(fileiobase.data_nd):
 
         # find the length of the third dimension
         f3 = "FDF" + str(int(dic["FDDIMORDER3"]))
-        lenZ = int(dic[f3 + "SIZE"])
+        quadrature_factor = [2, 1][int(dic[f3 + 'QUADFLAG'])]
+        if dic[f3 + 'QUADFLAG']:
+            lenZ = int(dic[f3 + 'FTSIZE'] * quadrature_factor)
+        else:
+            lenZ = int(dic[f3 + 'TDSIZE'] * quadrature_factor)
         fshape.insert(0, lenZ)   # insert as leading size of fshape
 
         # find the length of the fourth dimension
         f4 = "FDF" + str(int(dic["FDDIMORDER4"]))
-        lenA = int(dic[f4 + "SIZE"])
+        quadrature_factor = [2, 1][int(dic[f4 + 'QUADFLAG'])]
+        if dic[f4 + 'QUADFLAG']:
+            lenA = int(dic[f4 + 'FTSIZE'] * quadrature_factor)
+        else:
+            lenA = int(dic[f4 + 'TDSIZE'] * quadrature_factor)
         fshape.insert(0, lenA)   # insert as leading size of fshape
 
         # check that all files exist if fcheck is set
